#'Plot targets and histograms for illustrating accuracy and precision.
#'
#'Plot \dQuote{targets} and histograms for illustrating accuracy and precision.
#'The user can choose the number of points to sample.
#'
#'@param n Number of random points to put on each target and histogram.
#'@param r Number of rings (i.e., circles) on the target.
#'@param pts.col String indicating color to use for plotting points on the
#'target.
#'@param pts.trans Value between 0 and 1 indicating the transparency to use for
#'plotting points on the target.  The inverse of this value indicates how many
#'points must be overplotted before the full color is shown.
#'@param pts.cex Character expansion amount for points on target.
#'@param mns.col String indicating color to use for line marking the mean on
#'the targets and the histograms.
#'@param mns.pch Value indicating the plotting character to use for the mean on
#'the targets.
#'@param mns.lwd Value indicating the line width to use use for the mean on the
#'histograms.
#'@return None.  A plot is produced.
#'@keywords dynamic
#'@examples
#'accuracyPrecision()
#'accuracyPrecision(100,5)
#'
#'@export
accuracyPrecision <- function(n=50,r=4,pts.col="red",pts.trans=1,pts.cex=1,
                                       mns.col="blue",mns.pch=3,mns.lwd=2) {

  ## Internal function to make the bullseye target graph with random points on it.
  makeTarget<-function(r,px,py,t1,t2,pts.cex,pts.col,pts.trans,mns.col,mns.pch){  
    # Make the schematic plot with a bullseye in the middle
    plot(0,0,xlim=c(-r,r),ylim=c(-r,r),type="p",pch=19,axes=FALSE,yaxt="n",xaxt="n",xlab="",ylab="")  
    for (i in 1:r) {                          # Put on circles with integer radii to r
      cx <- seq(-i,i,by=0.005)                # Find x-coords of circle
      cyt <- sqrt(i^2-cx^2)                   # Find y-coords for top-half of the circle
      cyb <- -sqrt(i^2-cx^2)                  # Find y-coords for bottom-half of the circle
      clr <- ifelse(i==r,"grey30","grey80")   # Inner circles are ligher grey
      lines(cx,cyt,lwd=2,col=clr)             # Draw upper half of circle
      lines(cx,cyb,lwd=2,col=clr)             # Draw lower half of circle
    }
    col2 <- col2rgb(pts.col)/255
    pts.col <- rgb(col2[1],col2[2],col2[3],pts.trans)
    # Put random points on graph in red
    points(px,py,pch=19,col=pts.col,cex=pts.cex)                                
    # Put a blue plus at center of points (mx,my)
    points(mean(px),mean(py),pch=mns.pch,col=mns.col,cex=round(1.5*pts.cex,1))  
    mtext(t1,2)         # Label graphs
    mtext(t2,2,1.5)
  } ## end internal makeTarget

  ## Internal function to make the histogram of the random points.
  makeDist<-function(px,ylmts,brks,pts.col,pts.trans,mns.col,mns.lwd) {  
    col2 <- col2rgb(pts.col)/255
    pts.col <- rgb(col2[1],col2[2],col2[3],pts.trans)
    hist(px,main="",breaks=brks,axes=FALSE,ylim=ylmts,col=pts.col)  # Plot the histogram
    lines(c(mean(px),mean(px)),ylmts,lwd=mns.lwd,col=mns.col)       # Put blue vertical line at sample mean
    lines(c(0,0),c(-0.25,0),lwd=2,col="black")                      # Put black tick at true mean
    text(0,-0.25,"T",pos=1,xpd=TRUE,cex=1.5)
  } ## end internal makeDist()

  old.par <- par(mfcol=c(4,2),mar=c(1.5,3,1.5,0)); on.exit(par(old.par))
  ps <- 1/2                                                         # Standard deviation for precise situations
  is <- r/3                                                         # Standard deviation for imprecise situations
  ap.x <- rnorm(n,0,ps)
  ai.x <- rnorm(n,0,is)
  ip.x <- rnorm(n,1.5,ps)
  ii.x <- rnorm(n,1.5,is)

  makeTarget(r,ap.x,rnorm(n,0,ps),"Accurate","Precise",pts.cex,pts.col,pts.trans,mns.col,mns.pch) 
  makeTarget(r,ai.x,rnorm(n,0,is),"Accurate","Imprecise",pts.cex,pts.col,pts.trans,mns.col,mns.pch)
  makeTarget(r,ip.x,rnorm(n,-1.5,ps),"Inaccurate","Precise",pts.cex,pts.col,pts.trans,mns.col,mns.pch)
  makeTarget(r,ii.x,rnorm(n,-1.5,is),"Inaccurate","Imprecise",pts.cex,pts.col,pts.trans,mns.col,mns.pch)

  ylmts <- c(0,max(hist(ap.x,plot=FALSE)$counts,hist(ai.x,plot=FALSE)$counts,hist(ip.x,plot=FALSE)$counts,hist(ii.x,plot=FALSE)$counts))
  minbrk <- min(hist(ap.x,plot=FALSE)$breaks,hist(ai.x,plot=FALSE)$breaks,hist(ip.x,plot=FALSE)$breaks,hist(ii.x,plot=FALSE)$breaks)
  maxbrk <- max(hist(ap.x,plot=FALSE)$breaks,hist(ai.x,plot=FALSE)$breaks,hist(ip.x,plot=FALSE)$breaks,hist(ii.x,plot=FALSE)$breaks)
  brks <- seq(minbrk,maxbrk,by=0.5)
  par(mar=c(2,1,1,1))
  makeDist(ap.x,ylmts,brks,pts.col,pts.trans,mns.col,mns.lwd)       # Dist - Accurate and Precise
  makeDist(ai.x,ylmts,brks,pts.col,pts.trans,mns.col,mns.lwd)       # Dist - Accurate and Imprecise
  makeDist(ip.x,ylmts,brks,pts.col,pts.trans,mns.col,mns.lwd)       # Dist - Inaccurate and Precise
  makeDist(ii.x,ylmts,brks,pts.col,pts.trans,mns.col,mns.lwd)       # Dist - Inaccurate and Imprecise
}
